/* https://github.com/hustcc/placeholder.js/ */

var c, cc, rendered_attr = 'placeholder-rendered';

function createCanvas(opts) {
    if (!c || !cc) {
        c = document.createElement('canvas');
        cc = c.getContext('2d');
    }
    var canvas_width = parseInt(opts.a[0]), canvas_height = parseInt(opts.a[1]);
    c.width = canvas_width;
    c.height = canvas_height;
    cc.clearRect(0, 0, canvas_width, canvas_height);
    cc.fillStyle = opts.c;
    cc.fillRect(0, 0, canvas_width, canvas_height);
    cc.fillStyle = opts.d;
    cc.font = opts.e + ' normal ' + opts.f + ' ' + (opts.g || 100) + 'px ' + opts.h;
    var scale = 1.0;
    if (opts.g === '') {
        var width_limit = 0.7 * canvas_width,
                heigth_limit = 0.7 * canvas_height,
                text_width = cc.measureText(opts.b).width,
                text_height = 100;
        scale = Math.min(width_limit / text_width, heigth_limit / text_height);
    }
    cc.translate(canvas_width / 2, canvas_height / 2);
    cc.scale(scale, scale);
    cc.textAlign = 'center';
    cc.textBaseline = 'middle';

    cc.fillText(opts.b, 0, 0);
    return c;
}

function randomColor() {
    return '#' + ('00000' + (Math.random() * 0x1000000 << 0).toString(16)).slice(-6);
}

function prepareOpts(opts) {
    opts = opts || {};
    var size = opts.size || '128x128',
            text = opts.text || size,
            bgcolor = opts.bgcolor || randomColor(),
            color = opts.color || randomColor(),
            fstyle = opts.fstyle || 'normal',
            fweight = opts.fweight || 'bold',
            fsize = opts.fsize || '',
            ffamily = opts.ffamily || 'consolas',
            new_opts = {};

    size = size.split('x');
    if (size.length !== 2) {
        size = [128, 128];
    }
    new_opts.a = size;
    new_opts.b = text;
    new_opts.c = bgcolor;
    new_opts.d = color;
    new_opts.e = fstyle;
    new_opts.f = fweight;
    new_opts.g = fsize;
    new_opts.h = ffamily;
    opts = null;
    return new_opts;
}

function placeholderCanvas(opts) {
    opts = prepareOpts(opts);
    return createCanvas(opts);
}

function placeholder(opts) {
    return placeholderCanvas(opts).toDataURL();
}

function _getAttribute(node, name, defaultValue) {
    return node.getAttribute(name) || defaultValue;
}

function parseUrlOptions(url) {
    var opts = {}, parameters = url.split('&'), k_v;
    for (var i in parameters) {
        k_v = parameters[i].split('=');
        try {
            opts[k_v[0]] = decodeURIComponent(k_v[1]);
        } catch (e) {
            opts[k_v[0]] = k_v[1];
        }
    }
    return opts;
}

function render(force) {
    var image_eles = document.querySelectorAll('img.placeholder'), img, opts;
    for (var i = 0; i < image_eles.length; i++) {
        img = image_eles[i];
        if (force || (!_getAttribute(img, rendered_attr, ''))) {
            opts = parseUrlOptions(_getAttribute(img, 'options', ''));
            img.setAttribute('src', placeholder(opts));
            img.setAttribute(rendered_attr, '1');
        }
    }
}

render();
